from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '1.0.9'

PACKAGE_NAME = "gameboard"
DESCRIPTION = 'A 3D grid creation library for simulation and games.'
AUTHOR_NAME = "Alex"
AUTHOR_EMAIL = "sprocketerdev@gmail.com"
PROJECT_URL = "https://github.com/Sprocketer/gameboard"
# REQUIRED_PACKAGES = []
PROJECT_KEYWORDS = ['pypi', 'python']
# https://pypi.org/classifiers/
CLASSIFIERS = [
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows"]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author=AUTHOR_NAME,
    author_email=AUTHOR_EMAIL,
    description=DESCRIPTION,
    url = PROJECT_URL,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    # install_requires=REQUIRED_PACKAGES,
    keywords=PROJECT_KEYWORDS,
    classifiers=CLASSIFIERS
)



from numpy.typing import NDArray
from typing import Any

class grid():
    """# Grid object class.
    
    `grid_size: int`
        The size of the grid, must be an integer.
    
    `fill: bool`
        If True, the grid will be filled with numbers from 1 to grid_size^2, horizontally from left to right, top to bottom."""
    
    def __init__(self, grid_size: int, fill: bool = False):
        
        if not isinstance(grid_size, int):
            raise ValueError("grid_size must be an integer")
        if not isinstance(fill, bool):
            raise ValueError("fill must be a boolean")

        self.grid_size = grid_size
        self.edge_index = grid_size - 1
        
        self._data = []
        if fill == True:
            count = 0
            for _ in range(self.grid_size): # for each row
                row = []
                for _ in range(self.grid_size): # for each item
                    count += 1
                    row.append([count])
                self._data.append(row)
        else:
            for _ in range(self.grid_size): # for each row
                row = []
                for _ in range(self.grid_size): # for each item
                    row.append([])
                self._data.append(row)
    
    def __repr__(self):
        grid_str = ""
        for row in self._data:
            grid_str += str(row) + "," + "\n"*2
        grid_str = "[" + grid_str[:-3] + "]"
        return grid_str

    def __getitem__(self, pos: tuple[int, int] | list[int] | NDArray):
        y = self.edge_index - pos[1]
        x = pos[0]
        return self._data[y][x]

    def __iter__(self):
        for elem in self._data:
            yield elem
    
    def __setitem__(self, pos: tuple[int, int] | list[int] | NDArray, value: Any):
        y = self.edge_index - pos[1]
        x = pos[0]
        self._data[y][x] = value